package Source;

import java.awt.Point;
import javax.swing.JProgressBar;

public abstract class Grid implements java.io.Serializable {

    public PointList[][] grid;
    public int size;

    public Grid(int size) {
        this.size = size;
        grid = new PointList[size][size];
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
                grid[i][j] = new PointList();
            }
        }
    }

    public Point calculateGridPosition(EntityPoint p) {
        return new Point((int) Math.floor(p.y), (int) Math.floor(p.x));
    }

    public Point addPoint(EntityPoint p) {
        grid[calculateGridPosition(p).x][calculateGridPosition(p).y].list.add(p);
        return calculateGridPosition(p);
    }

    public boolean removePoint(EntityPoint p) {
        return grid[calculateGridPosition(p).x][calculateGridPosition(p).y].list.remove(p);
    }

    public abstract double distance(EntityPoint p1, EntityPoint p2);

    public abstract EntityPoint mapping(EntityPoint p);

    public abstract EntityPoint mapping(EntityPoint p, int c);

    public abstract Point mapping(Point p);

    public abstract EntityPoint translate(EntityPoint p, EntityPoint c, double factor);

    public PointList getByProximity(EntityPoint rp, double radius) {
        PointList points = new PointList();
        int top = (int) (Math.floor(rp.y - radius));
        int left = (int) (Math.floor(rp.x - radius));

        for (int i = top; i <= top + 2 * radius; i++) {
            for (int j = left; j <= left + 2 * radius; j++) {
                for (int k = 0; k < grid[mapping(new Point(j, i)).y][mapping(new Point(j, i)).x].list.size(); k++) {
                    if (distance(rp, grid[mapping(new Point(j, i)).y][mapping(new Point(j, i)).x].list.get(k)) < radius) {
                        points.list.add(grid[mapping(new Point(j, i)).y][mapping(new Point(j, i)).x].list.get(k));
                    }
                }
            }
        }

        return points;
    }

    public PointList getBySquare(EntityPoint rp, double side) {
        PointList points = new PointList();
        int top = (int) (Math.floor(rp.y - side / 2));
        int left = (int) (Math.floor(rp.x - side / 2));

        for (int i = top; i <= top + side; i++) {
            for (int j = left; j <= left + side; j++) {
                for (int k = 0; k < grid[mapping(new Point(j, i)).y][mapping(new Point(j, i)).x].list.size(); k++) {
                    for (int c = 0; c < 9; c++) {
                        if (inSquare(mapping(grid[mapping(new Point(j, i)).y][mapping(new Point(j, i)).x].list.get(k), c), rp, side)) {
                            points.list.add(grid[mapping(new Point(j, i)).y][mapping(new Point(j, i)).x].list.get(k));
                        }
                    }
                }
            }
        }
        return points;
    }

    public boolean inSquare(EntityPoint p, EntityPoint c, double side) {
        if (p.x > c.x - side / 2 && p.x < c.x + side / 2 && p.y > c.y - side / 2 && p.y < c.y + side / 2) {
            return true;
        } else {
            return false;
        }
    }

    public PointList getByTriangle(EntityPoint rp, double side) {
        PointList points = new PointList();
        int top = (int) (Math.floor(rp.y - side / 2));
        int left = (int) (Math.floor(rp.x - side / 2));

        for (int i = top; i <= top + side; i++) {
            for (int j = left; j <= left + side; j++) {
                for (int k = 0; k < grid[mapping(new Point(j, i)).y][mapping(new Point(j, i)).x].list.size(); k++) {
                    for (int c = 0; c < 9; c++) {
                        if (inTriangle(mapping(grid[mapping(new Point(j, i)).y][mapping(new Point(j, i)).x].list.get(k), c), rp, side)) {
                            points.list.add(grid[mapping(new Point(j, i)).y][mapping(new Point(j, i)).x].list.get(k));
                        }
                    }
                }
            }
        }
        return points;
    }

    public boolean inTriangle(EntityPoint p, EntityPoint cc, double side) {
        // Compute vectors     
        EntityPoint a = new EntityPoint(cc.x, cc.y - side * Math.sqrt(3) / 3);
        EntityPoint b = new EntityPoint(cc.x - side / 2, cc.y + side * Math.sqrt(3) / 6);
        EntityPoint c = new EntityPoint(cc.x + side / 2, cc.y + side * Math.sqrt(3) / 6);

        EntityPoint v0 = new EntityPoint(c.x - a.x, c.y - a.y);
        EntityPoint v1 = new EntityPoint(b.x - a.x, b.y - a.y);
        EntityPoint v2 = new EntityPoint(p.x - a.x, p.y - a.y);

        // Compute dot products
        double dot00 = v0.x * v0.x + v0.y * v0.y;
        double dot01 = v0.x * v1.x + v0.y * v1.y;
        double dot02 = v0.x * v2.x + v0.y * v2.y;
        double dot11 = v1.x * v1.x + v1.y * v1.y;
        double dot12 = v1.x * v2.x + v1.y * v2.y;

        // Compute barycentric coordinates
        double invDenom = 1 / (dot00 * dot11 - dot01 * dot01);
        double u = (dot11 * dot02 - dot01 * dot12) * invDenom;
        double v = (dot00 * dot12 - dot01 * dot02) * invDenom;

        // Check if point is in triangle
        return (u >= 0) && (v >= 0) && (u + v < 1);
    }

    public void emptyGrid() {
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
                grid[i][j] = new PointList();
            }
        }
    }

    public void generateRandom(int treeCount, double distance, JProgressBar progressBar) {
        progressBar.setString("Generating " + treeCount + " random entities..");
        progressBar.setValue(0);
        emptyGrid();
        int count = 0;
        while (count < treeCount) {
            EntityPoint p = new EntityPoint(size);
            if (getByProximity(p, distance).list.isEmpty()) {
                addPoint(p);
                count++;
            }
            progressBar.setValue((100 * count) / treeCount);
            progressBar.paint(progressBar.getGraphics());
        }
        progressBar.setString("Generating " + treeCount + " random entities.. DONE");
    }

    public int generateUniform(int treeCount) {
        emptyGrid();
        int side = (int) Math.floor(Math.sqrt(treeCount));

        for (int i = 0; i < side; i++) {
            for (int j = 0; j < side; j++) {
                EntityPoint p = new EntityPoint();
                p.x = (j + 0.5) * (size / side);
                p.y = (i + 0.5) * (size / side);
                addPoint(p);
            }
        }
        return side * side;
    }

    public PointList getPointList() {
        PointList list = new PointList();
        for (int i = 0; i < size; i++) {
            for (int j = 0; j < size; j++) {
                for (int k = 0; k < grid[i][j].list.size(); k++) {
                    list.list.add(grid[i][j].list.get(k));
                }
            }
        }
        return list;

    }
}
