package Source;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JProgressBar;

public class Sample implements java.io.Serializable {

    long id;
    Forest forest;
    int size;
    double area;                  // in ha
    String shape;                 // eg "circle", "square"   
    String layout;
    int repetition;
    Grid grid;
    PointList[] plots;

    public Sample(Forest forest, String sampleShape, double sampleArea, int sampleCount, String layout, int repetition) {
        this.id = System.currentTimeMillis();
        this.forest = forest;
        this.size = sampleCount;
        this.area = sampleArea;
        this.shape = sampleShape;
        this.layout = layout;
        this.repetition = repetition;
        if (forest.folding.equals("Torus")) {
            grid = new TorusGrid(forest.forestSide);
        } else if (forest.folding.equals("Sphere")) {
            grid = new SphereGrid(forest.forestSide);
        } else {//default
            grid = new TorusGrid(forest.forestSide);
        }
    }

    public void saveToFile(File file) {
        try {
            FileOutputStream fileOut = new FileOutputStream(file);
            ObjectOutputStream out = new ObjectOutputStream(fileOut);
            out.writeObject(this);
            out.close();
            fileOut.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public String loadFromFile(File file) {
        try {
            FileInputStream fileIn = new FileInputStream(file);
            ObjectInputStream in = new ObjectInputStream(fileIn);
            loadFromSample((Sample) in.readObject());
            in.close();
            fileIn.close();
        } catch (IOException | ClassNotFoundException i) {
            return "I/O or Class error!";
        }
        return "Sample load succesful!";
    }

    public void loadFromSample(Sample sample) {
        this.forest = sample.forest;
        this.size = sample.size;
        this.area = sample.area;
        this.shape = sample.shape;
        this.layout = sample.layout;
        this.grid = sample.grid;
    }

    public void outputToFile(File file, JProgressBar progressBar) {
        progressBar.setString("Writing sample to file..");
        progressBar.setValue(0);
        FileWriter fw = null;
        try {
            if (file == null) {
                file = new File(System.getProperty("user.dir") + "/SampleOutput/" + this.id + ".csv");
            }
            insertSampleID();
            boolean fileExists = false;
            if (file.exists()) {
                fileExists = true;
            }
            fw = new FileWriter(file, true);
            BufferedWriter out = new BufferedWriter(fw);
            // if new file, write header
            if (!fileExists) {
                out.write("sampleID,forestID,X,Y,Weibull,plot\n");
            }

            //write the forest
            for (int i = 0; i < plots.length; i++) {
                for (EntityPoint p : plots[i].list) {
                    out.write(id + "," + forest.id + "," + p.x + "," + p.y + "," + p.weibull + "," + i + ",\n");
                }
                progressBar.setValue((100 * i) / plots.length);
                progressBar.paint(progressBar.getGraphics());
            }
            out.close();
            progressBar.setValue(100);
            progressBar.setString("Writing sample to file..DONE");
        } catch (IOException ex) {
            Logger.getLogger(Sample.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            try {
                fw.close();
            } catch (IOException ex) {
                Logger.getLogger(Sample.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    public void insertSampleID() {
        boolean exists = false;
        try {
            //SampleID exists?
            FileInputStream fstream = new FileInputStream(System.getProperty("user.dir") + "/Registers/sample_register.csv");
            DataInputStream in = new DataInputStream(fstream);
            BufferedReader br = new BufferedReader(new InputStreamReader(in));
            String line;
            while ((line = br.readLine()) != null) {
                if (line.indexOf("" + id) != -1) {
                    exists = true;
                }
            }
            in.close();

            //if does not exist add it 
            FileWriter fw = new FileWriter(System.getProperty("user.dir") + "/Registers/sample_register.csv", true);
            BufferedWriter out = new BufferedWriter(fw);
            if (!exists) {
                out.write(id + "," + size + "," + shape + "," + area + "," + layout + "," + repetition + "\n");
            }
            out.close();
        } catch (Exception e) {//Catch exception if any
            System.err.println("Error: " + e.getMessage());
        }
    }
}
