package Source;


import java.awt.Point;

public class SphereGrid extends Grid {

    public SphereGrid(int size) {
        super(size);
    }

    public double distance(EntityPoint p1, EntityPoint p2) {
        double dist;
        dist = dist(p1, p2);

        for (int i = 1; i < 10; i++) {
            if (dist > dist(p1, mapping(p2, i))) {
                dist = dist(p1, mapping(p2, i));
            }
        }

        return dist;
    }

    public EntityPoint distancePoint(EntityPoint p1, EntityPoint p2) {
        double dist;
        dist = dist(p1, p2);
        EntityPoint dp = p2;

        for (int i = 1; i < 10; i++) {
            if (dist > dist(p1, mapping(p2, i))) {
                dist = dist(p1, mapping(p2, i));
                dp = mapping(p2, i);
            }
        }

        return dp;
    }

    public double dist(EntityPoint p1, EntityPoint p2) {
        return Math.sqrt(Math.pow(p1.x - p2.x, 2) + Math.pow(p1.y - p2.y, 2));
    }

    public EntityPoint mapping(EntityPoint p, int quadrant) {
        double factor;
        if (p.x < 0.5 * size) {
            factor = 0.5;
        } else {
            factor = 1.5;
        }

        switch (quadrant) {
            case 1:
                return new EntityPoint(p.x + (0-factor) * size, -p.y);
            case 2:
                return new EntityPoint(p.x + (1-factor) * size, -p.y);
            case 3:
                return new EntityPoint(p.x + (2-factor) * size, -p.y);
            case 4:
                return new EntityPoint(p.x + size, p.y);
            case 5:
                return new EntityPoint(p.x + (2-factor) * size, p.y + 2 * (size - p.y));
            case 6:
                return new EntityPoint(p.x + (1-factor) * size, p.y + 2 * (size - p.y));
            case 7:
                return new EntityPoint(p.x + (0-factor) * size, p.y + 2 * (size - p.y));
            case 8:
                return new EntityPoint(p.x - size, p.y);
            default:
                return new EntityPoint(p.x, p.y);
        }
    }

    public EntityPoint mapping(EntityPoint p) {
        
        EntityPoint mp = new EntityPoint(p.x, p.y);
        
        while(mp.x<0)
            mp.x+=size;
        while (mp.x>=size)
            mp.x-=size;        
        while (mp.y<0)
            mp.y+=2*size;        
        while (mp.y>=size)
            mp.y-=2*size;
        
        if(mp.y<0)
        {
            mp.y = -mp.y;
            if(mp.x<0.5*size)
            {
            mp.x+=.5*size;
            }
            else
            mp.x-=.5*size;
        }
               
        return mp;
    }

    public Point mapping(Point p) {
        EntityPoint mp = new EntityPoint(p.x, p.y);
        mp = mapping(mp);
        if (mp.y==size) mp.y-=1;
        return new Point((int) Math.floor(mp.x), (int) Math.floor(mp.y));
    }

    public EntityPoint translate(EntityPoint p, EntityPoint c, double factor) {
        EntityPoint dp = distancePoint(p, c);

        EntityPoint tp = new EntityPoint(p.x, p.y);
        tp.x += factor * (dp.x - p.x);
        tp.y += factor * (dp.y - p.y);

        return mapping(tp);
    }
}
