package Source;


import java.awt.Point;

public class TorusGrid extends Grid {

    public TorusGrid(int size) {
        super(size);
    }

    public double distance(EntityPoint p1, EntityPoint p2) {
        double distx, disty;
        if (Math.pow(p1.x - p2.x, 2) < Math.pow(size - Math.abs(p1.x - p2.x), 2)) {
            distx = Math.pow(p1.x - p2.x, 2);
        } else {
            distx = Math.pow(size - Math.abs(p1.x - p2.x), 2);
        }

        if (Math.pow(p1.y - p2.y, 2) < Math.pow(size - Math.abs(p1.y - p2.y), 2)) {
            disty = Math.pow(p1.y - p2.y, 2);
        } else {
            disty = Math.pow(size - Math.abs(p1.y - p2.y), 2);
        }
        return Math.sqrt(distx + disty);
    }

    public EntityPoint mapping(EntityPoint p) {
        return new EntityPoint(mapping(p.x), mapping(p.y));
    }

    public Point mapping(Point p) {
        return new Point(mapping(p.x), mapping(p.y));

    }

    public double mapping(double n) {
        if (n % size < 0) {
            return (n % size) + size;
        }

        return n % size;
    }

    public EntityPoint mapping(EntityPoint p, int c) {
        EntityPoint mp = new EntityPoint(p.x, p.y);
        mp = mapping(mp);

        switch (c) {
            case 1:
                return mp.offset(-size, -size);
            case 2:
                return mp.offset(0, -size);
            case 3:
                return mp.offset(size, -size);
            case 4:
                return mp.offset(size, 0);
            case 5:
                return mp.offset(size, size);
            case 6:
                return mp.offset(0, size);
            case 7:
                return mp.offset(-size, size);
            case 8:
                return mp.offset(-size, 0);
            default:
                return mp;
        }
    }

    public int mapping(int n) {
        if (n % size < 0) {
            return (n % size) + size;
        }

        return n % size;
    }

    public EntityPoint translate(EntityPoint p, EntityPoint c, double factor) {
        EntityPoint tp = new EntityPoint();

        //translate X
        if ((p.x < c.x) && (c.x - p.x > size / 2.0)) {
            tp.x = mapping(p.x - factor * (size - c.x + p.x));
        }
        if ((p.x >= c.x) && (p.x - c.x > size / 2.0)) {
            tp.x = mapping(p.x + factor * (size + c.x - p.x));
        }
        if ((p.x < c.x) && (c.x - p.x <= size / 2.0)) {
            tp.x = mapping(p.x + factor * (c.x - p.x));
        }
        if ((p.x >= c.x) && (p.x - c.x <= size / 2.0)) {
            tp.x = mapping(p.x - factor * (p.x - c.x));
        }

        //translate Y
        if ((p.y < c.y) && (c.y - p.y > size / 2.0)) {
            tp.y = mapping(p.y - factor * (size - c.y + p.y));
        }
        if ((p.y >= c.y) && (p.y - c.y > size / 2.0)) {
            tp.y = mapping(p.y + factor * (size + c.y - p.y));
        }
        if ((p.y < c.y) && (c.y - p.y <= size / 2.0)) {
            tp.y = mapping(p.y + factor * (c.y - p.y));
        }
        if ((p.y >= c.y) && (p.y - c.y <= size / 2.0)) {
            tp.y = mapping(p.y - factor * (p.y - c.y));
        }
        return tp;
    }    
}
